;==========================================================
; PSMFILL.LSP   Rev 1
; Copyright 1995 by Looking Glass Microproducts
;==========================================================
; Apply the PSFILL command to selected entities.
;
(Setq
   psmfill:pattern "." ; last fill pattern used
   psmfill:params  Nil ; PostScript Fill Pattern Parameters
)
(Defun c:psmfill (/ old_error error filter cbc_getstring cbc_mapset
                  cbc_notrans cbc_ltrim cbc_rtrim cbc_trim cbc_getreal
                  cbc_getint parse bumpno bump get_params get_one_arg
                  get_args ss1 bumpno undoit)
   ;=======================================================
   ; Selection set filter closed, 2D Polylines
   ;=======================================================
   (Setq
      filter (List
                '(0 . "POLYLINE")
                '(-4 . "&")
                '(70 . 1)
                '(-4 . "<not")
                '(-4 . "&")
                '(70 . 88)
                '(-4 . "not>")
             )
   )
   ;=======================================================
   ; Error Handler
   ;=======================================================
   (Defun error (s)
      (If (Not
             (Member
                S
                '("Function cancelled" "console break")
             )
          )
         (Princ s)
      )
      (If fh (Progn (Close fh) (Setq fh Nil)))
      (Setq *Error* old_error)
      (Princ)
   )
   ;=======================================================
   ; Get String with default
   ;=======================================================
   (Defun cbc_getstring (cr prmpt default / temp)
      (Cond
         ((=
             ""
             (Setq
                temp (GetString
                        cr
                        (If (/= "" default)
                           (Strcat prmpt " <" default ">: ")
                           (Strcat prmpt ": ")
                        )
                     )
             )
          )
            default
         )
         (T temp)
      )
   )
   ;=======================================================
   ; Apply func to every entity in ss
   ;=======================================================
   (Defun cbc_mapset (func ss / n ename)
      (If ss
         (Progn
            (Setq func (Eval func) n (SsLength ss))
            (While (> n 0)
               (Setq n (1- n) ename (SsName ss n))
               (func ename)
            )
         )
      )
   )
   ;==========================================================
   ; Disallow transparent invocation of routine.
   ;==========================================================
   (Defun cbc_notrans ()
      (Cond
         ((Zerop
             (LogAnd (GetVar "cmdactive") (+ 1 2 8))
          )
         )
         ((Alert
             "This command may not be invoked transparently."
          )
         )
      )
   )
   ;==========================================================
   ; Trim Leading Spaces from string
   ;==========================================================
   (Defun cbc_ltrim (astring)
      (While (WCmatch astring " *")
         (Setq astring (Substr astring 2))
      )
      astring
   )
   ;==========================================================
   ; Trim Trailing Spaces from string
   ;==========================================================
   (Defun cbc_rtrim (astring)
      (While (WCmatch astring "* ")
         (Setq
            astring (Substr astring 1 (1- (Strlen astring)))
         )
      )
      astring
   )
   ;==========================================================
   ; Trim Leading and Trailing Spaces from string
   ;==========================================================
   (Defun cbc_trim (astring)
      (cbc_rtrim (cbc_ltrim astring))
   )
   ;==========================================================
   ; GetReal with Default
   ;==========================================================
   (Defun cbc_getreal (prmpt default)
      (Setq
         prmpt (Strcat
                  prmpt
                  (If default
                     (Strcat " <" (RtoS default 2) ">")
                     ""
                  )
                  ": "
               )
      )
      (Cond ((GetReal prmpt)) (default))
   )
   ;==========================================================
   ; Get Integer with default
   ;==========================================================
   (Defun cbc_getint (prmpt default)
      (Cond
         ((GetInt
             (Strcat prmpt " <" (ItoA default) ">: ")
          )
         )
         (default)
      )
   )
   ;=======================================================
   ; Parse parameters from acad.psf
   ;=======================================================
   (Defun parse (aline / pattern copyto nparams params)
      ;====================================================
      ; Copy aline upto delim, strip delim from aline
      ;====================================================
      (Defun copyto (delim / c token)
         (Setq token "")
         (While (And
                   (/= delim (Setq c (Substr aline 1 1)))
                   (/= "" c)
                )
            (If (/= " " c)
               (Setq token (Strcat token c))
            )
            (Setq aline (Substr aline 2))
         )
         (Setq aline (Substr aline 2))
         token
      )
      ;=============================
      ; Start of Parse
      ;=============================
      (copyto "%")
      (Setq
         pattern (copyto ",")
         nparams (AtoI (copyto ","))
         params  (List nparams pattern (Strcase pattern))
      )
      (While (/= "" aline)
         (Setq
            params (Cons
                      (Cons (copyto "=") (Read (copyto ",")))
                      params
                   )
         )
      )
      (Reverse params)
   )
   ;=======================================================
   ; Entertainment
   ;=======================================================
   (Setq bumpno 0)
   (Defun bump ()
      (Setq bumpno (LogAnd (1+ bumpno) 3))
      (Princ
         (Nth bumpno '("\r\\" "\r-" "\r/" "\r|"))
      )
   )
   ;=======================================================
   ; Read parameters from acad.psf
   ;=======================================================
   (Defun get_params (/ fname fullname fh params lbuf params)
      (Setq fname "ACAD.PSF")
      (Cond
         ((Null (Setq fullname (FindFile fname)))
            (Alert
               (Strcat fname " -- Can't find file.")
            )
         )
         ((Null (Setq fh (Open fullname "r")))
            (Alert
               (Strcat fname " -- Can't open file.")
            )
         )
         (T
            (Princ
               (Strcat "\n  Reading " fullname "...\r")
            )
            (While (Setq lbuf (Read-Line fh))
               (If (And
                      (= (Strcase lbuf) "%@FILL")
                      (Setq lbuf (Read-Line fh))
                   )
                  (Progn
                     (bump)
                     (Setq params (Cons (parse lbuf) params))
                  )
               )
            )
            (Close fh)
         )
      )
      (Reverse params)
   )
   ;=======================================================
   ; Get One Arg
   ;=======================================================
   (Defun get_one_arg (a)
      (Cond
         ((Atom a) a)
         ((= 'Real (Type (CDR a)))
            (Cons
               (CAR a)
               (cbc_getreal
                  (Strcat "\n" (CAR a))
                  (CDR a)
               )
            )
         )
         ((= 'Int (Type (CDR a)))
            (Cons
               (CAR a)
               (cbc_getint
                  (Strcat "\n" (CAR a))
                  (CDR a)
               )
            )
         )
         ((T a))
      )
   )
   ;=======================================================
   ; Get Args
   ;=======================================================
   (Defun get_args (/ again pattern prmpt patterns old_params new_params
                    args list_patterns)
      ;===================================
      ; Neatly list patterns from acad.psf
      ;===================================
      (Defun list_patterns (/ temp)
         (TextPage)
         (Setq temp "\n")
         (ForEach p patterns
            (Setq temp (Strcat temp p "  "))
            (If (> (Strlen temp) 70)
               (Progn (Princ temp) (Setq temp "\n"))
            )
         )
         (If (/= temp "\n") (Princ temp))
      )
      ;=====================
      ; start of get_args
      ;=====================
      (Setq
         patterns (acad_strlsort
                     (MapCar 'CADR psmfill:params)
                  )
         prmpt    "\nPostScript fill pattern (. = none) or ?"
         again    T
      )
      (While again
         (Setq
            pattern (cbc_GetString
                       Nil
                       prmpt
                       psmfill:pattern
                    )
         )
         (Cond
            ((= "?" pattern) (list_patterns))
            ((= "." pattern)
               (Setq
                  old_params      '("." "." 0)
                  psmfill:pattern pattern
                  again           Nil
               )
            )
            ((Setq
                old_params (Assoc (Strcase pattern) psmfill:params)
             )
               (Setq
                  psmfill:pattern (CADR old_params)
                  again           Nil
               )
            )
            ((And
                (WCmatch pattern "`**")
                (Setq
                   old_params (Assoc
                                 (Strcase (Substr pattern 2))
                                 psmfill:params
                              )
                )
             )
               (Setq
                  psmfill:pattern (Strcat "*" (CADR old_params))
                  again           Nil
               )
            )
            (T
               (Princ
                  (Strcat
                     "\nUnknown PostScript Fill Pattern: "
                     (Strcase pattern)
                  )
               )
            )
         )
      )
      (If old_params
         (Setq
            new_params     (MapCar 'get_one_arg old_params)
            psmfill:params (Subst
                              new_params
                              old_params
                              psmfill:params
                           )
            args           (Cons
                              psmfill:pattern
                              (MapCar 'CDR (CDDDR new_params))
                           )
         )
      )
      args
   )
   ;=======================================================
   ; Main routine.
   ;=======================================================
   (Setq old_error *Error* *Error* error)
   (Setvar "cmdecho" 0)
   (Setq ss1 (SsGet "i" filter))
   (Cond
      ((Not (cbc_notrans)))
      ((Progn
          (If ss1
             (Progn
                (Command ".select" ss1 "")
                (Princ
                   (Strcat
                      (ItoA (SsLength ss1))
                      " selected."
                   )
                )
             )
             (Setq ss1 (SsGet filter))
          )
          (Null ss1)
       )
      )
      ((And
          (Not c:psfill)
          (Not (Xload "acadapp" Nil))
       )
         (If (WCmatch (GetVar "platform") "* DOS *")
            (Alert
               "ACADAPP.EXP -- Cannot Xload file."
            )
            (Alert
               "ACADAPP.EXE -- Cannot Xload file."
            )
         )
      )
      ((And
          (Not psmfill:params)
          (Not (Setq psmfill:params (get_params)))
       )
      )
      ((Not (Setq args (get_args))))
      (T
         (Defun one_psfill (ename)
            (Apply 'c:psfill (Cons ename args))
         )
         (cbc_mapset 'one_psfill ss1)
      )
   )
   (Setq ss1 Nil *Error* old_error)
   (Princ)
)
(Princ
   (Strcat
      "\nPSMFILL.LSP -- (c) 1995 "
      "by Looking Glass Microproducts"
   )
)
(Princ)
